# Copyright (C) 2020-2021 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import unittest

from pontos.version import safe_version


class SafeVersionTestCase(unittest.TestCase):
    def test_dev_versions(self):
        self.assertEqual(safe_version('1.2.3dev'), '1.2.3.dev0')
        self.assertEqual(safe_version('1.2.3dev1'), '1.2.3.dev1')
        self.assertEqual(safe_version('1.2.3.dev'), '1.2.3.dev0')

    def test_alpha_versions(self):
        self.assertEqual(safe_version('1.2.3alpha'), '1.2.3a0')
        self.assertEqual(safe_version('1.2.3.alpha'), '1.2.3a0')
        self.assertEqual(safe_version('1.2.3a'), '1.2.3a0')
        self.assertEqual(safe_version('1.2.3.a1'), '1.2.3a1')
        self.assertEqual(safe_version('1.2.3a1'), '1.2.3a1')

    def test_beta_versions(self):
        self.assertEqual(safe_version('1.2.3beta'), '1.2.3b0')
        self.assertEqual(safe_version('1.2.3.beta'), '1.2.3b0')
        self.assertEqual(safe_version('1.2.3b'), '1.2.3b0')
        self.assertEqual(safe_version('1.2.3.b1'), '1.2.3b1')
        self.assertEqual(safe_version('1.2.3b1'), '1.2.3b1')

    def test_caldav_versions(self):
        self.assertEqual(safe_version('22.04'), '22.4')
        self.assertEqual(safe_version('22.4'), '22.4')
        self.assertEqual(safe_version('22.10'), '22.10')
        self.assertEqual(safe_version('22.04dev1'), '22.4.dev1')
        self.assertEqual(safe_version('22.10dev1'), '22.10.dev1')

    def test_release_versions(self):
        self.assertEqual(safe_version('1'), '1')
        self.assertEqual(safe_version('1.2'), '1.2')
        self.assertEqual(safe_version('1.2.3'), '1.2.3')
        self.assertEqual(safe_version('22.4'), '22.4')



# -*- coding: utf-8 -*-
# Copyright (C) 2021 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import datetime
from pathlib import Path
import sys

from packaging.version import Version, InvalidVersion

from pontos.version import (
    PontosVersionCommand,
    CMakeVersionCommand,
    VersionError,
)


def get_next_dev_version(release_version: str) -> str:
    """Get the next dev Version from a valid version"""
    # will be a dev1 version
    try:
        release_version_obj = Version(release_version)
        next_version_obj = Version(
            f'{str(release_version_obj.major)}.'
            f'{str(release_version_obj.minor)}.'
            f'{str(release_version_obj.micro + 1)}.dev1'
        )
        return str(next_version_obj)
    except InvalidVersion as e:
        raise (VersionError(e)) from None


def get_current_version() -> str:
    """Get the current Version from a pyproject.toml or
    a CMakeLists.txt file"""

    available_cmds = [
        ('CMakeLists.txt', CMakeVersionCommand),
        ('pyproject.toml', PontosVersionCommand),
    ]
    for file_name, cmd in available_cmds:
        project_definition_path = Path.cwd() / file_name
        if project_definition_path.exists():
            current_version: str = cmd().get_current_version()
            return current_version

    print("No project settings file found")
    sys.exit(1)


def calculate_calendar_version() -> str:
    """find the correct next calendar version by checking latest version and
    the today's date"""

    current_version_str: str = get_current_version()
    current_version = Version(current_version_str)

    today = datetime.date.today()

    if (
        current_version.major < today.year % 100
        or current_version.minor < today.month
    ):
        release_version = Version(
            f'{str(today.year  % 100)}.{str(today.month)}.0'
        )
        return str(release_version)
    elif (
        current_version.major == today.year % 100
        and current_version.minor == today.month
    ):
        if not current_version.dev:
            release_version = Version(
                f'{str(today.year  % 100)}.{str(today.month)}.'
                f'{str(current_version.micro + 1)}'
            )
        else:
            release_version = Version(
                f'{str(today.year  % 100)}.{str(today.month)}.'
                f'{str(current_version.micro)}'
            )
        return str(release_version)
    else:
        print(
            f"'{str(current_version)}' is higher than "
            f"'{str(today.year  % 100)}.{str(today.month)}'."
        )
        sys.exit(1)



# Copyright (C) 2020-2021 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import unittest

from pontos.version import strip_version


class StripVersionTestCase(unittest.TestCase):
    def test_version_string_without_v(self):
        self.assertEqual(strip_version('1.2.3'), '1.2.3')
        self.assertEqual(strip_version('1.2.3dev'), '1.2.3dev')

    def test_version_string_with_v(self):
        self.assertEqual(strip_version('v1.2.3'), '1.2.3')
        self.assertEqual(strip_version('v1.2.3dev'), '1.2.3dev')



# -*- coding: utf-8 -*-
# Copyright (C) 2020-2021 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import unittest

from pathlib import Path
from unittest.mock import MagicMock

from pontos.version import get_version_from_pyproject_toml, VersionError


class GetVersionFromPyprojectTomlTestCase(unittest.TestCase):
    def test_pyproject_toml_file_not_exists(self):
        fake_path_class = MagicMock(spec=Path)
        fake_path = fake_path_class.return_value
        fake_path.__str__.return_value = 'pyproject.toml'
        fake_path.exists.return_value = False

        with self.assertRaisesRegex(
            VersionError, 'pyproject.toml file not found'
        ):
            get_version_from_pyproject_toml(fake_path)

        fake_path.exists.assert_called_with()

    def test_no_poerty_section(self):
        fake_path_class = MagicMock(spec=Path)
        fake_path = fake_path_class.return_value
        fake_path.__str__.return_value = 'pyproject.toml'
        fake_path.exists.return_value = True
        fake_path.read_text.return_value = ''

        with self.assertRaisesRegex(
            VersionError, 'Version information not found in pyproject.toml file'
        ):
            get_version_from_pyproject_toml(fake_path)

        fake_path.exists.assert_called_with()
        fake_path.read_text.assert_called_with()

    def test_empty_poerty_section(self):
        fake_path_class = MagicMock(spec=Path)
        fake_path = fake_path_class.return_value
        fake_path.__str__.return_value = 'pyproject.toml'
        fake_path.exists.return_value = True
        fake_path.read_text.return_value = '[tool.poetry]'

        with self.assertRaisesRegex(
            VersionError, 'Version information not found in pyproject.toml file'
        ):
            get_version_from_pyproject_toml(fake_path)

        fake_path.exists.assert_called_with()
        fake_path.read_text.assert_called_with()

    def test_get_version(self):
        fake_path_class = MagicMock(spec=Path)
        fake_path = fake_path_class.return_value
        fake_path.__str__.return_value = 'pyproject.toml'
        fake_path.exists.return_value = True
        fake_path.read_text.return_value = '[tool.poetry]\nversion = "1.2.3"'

        version = get_version_from_pyproject_toml(fake_path)

        self.assertEqual(version, '1.2.3')

        fake_path.exists.assert_called_with()
        fake_path.read_text.assert_called_with()



# Copyright (C) 2020-2021 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import unittest

from pathlib import Path
from unittest.mock import MagicMock

from pontos.version import VersionCommand


class UpdateVersionFileTestCase(unittest.TestCase):
    def test_update_version_file(self):
        fake_path_class = MagicMock(spec=Path)
        fake_path = fake_path_class.return_value

        cmd = VersionCommand(version_file_path=fake_path)
        cmd.update_version_file('22.04dev1')

        text = fake_path.write_text.call_args[0][0]

        *_, version_line, _last_line = text.split('\n')

        self.assertEqual(version_line, '__version__ = "22.4.dev1"')



# Copyright (C) 2020 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import re
from typing import Tuple, List
from datetime import date


class ChangelogError(Exception):
    """
    Some error has occurred during changelog handling
    """


__UNRELEASED_MATCHER = re.compile('unreleased', re.IGNORECASE)
__MASTER_MATCHER = re.compile('master|HEAD')

__UNRELEASED_SKELETON = """## [Unreleased]
### Added
### Changed
### Deprecated
### Removed
### Fixed

[Unreleased]: https://github.com/{}/{}/compare/{}...HEAD


"""


def add_skeleton(
    markdown: str,
    new_version: str,
    project_name: str,
    git_tag_prefix: str = 'v',
    git_space: str = 'greenbone',
) -> str:
    git_tag = '{}{}'.format(git_tag_prefix, new_version)
    tokens = _tokenize(markdown)
    updated_markdown = ''

    for tt, _, tc in tokens:
        if tt == 'heading' and new_version in tc:
            prepared_skeleton = __UNRELEASED_SKELETON.format(
                git_space, project_name, git_tag
            )
            updated_markdown += prepared_skeleton + tc
        else:
            updated_markdown += tc

    return updated_markdown


def update(
    markdown: str,
    new_version: str,
    git_tag_prefix: str = 'v',
    containing_version: str = None,
) -> Tuple[str, str]:
    """
    update tokenizes CHANGELOG.md and if a version is given it changes
    unreleased headline and link to given version.

    returns updated markdown and change log for further processing.
    """

    git_tag = '{}{}'.format(git_tag_prefix, new_version)
    tokens = _tokenize(markdown)
    unreleased_heading_count = 0
    changelog = ''
    updated_markdown = ''
    current_state = []
    previous_state = []
    unreleased = []

    for tt, hc, tc in tokens:
        previous_state = current_state.copy()

        if tt == 'unreleased':
            if (
                containing_version and containing_version in tc
            ) or not containing_version:
                unreleased_heading_count = hc
                current_state = [tt]
        elif (
            tt == 'heading'
            and unreleased_heading_count > 0
            and hc <= unreleased_heading_count
        ):
            current_state = []

        if 'unreleased' not in current_state and 'unreleased' in previous_state:
            changelog = _prepare_changelog(unreleased, new_version, git_tag)
            updated_markdown += changelog

        if 'unreleased' not in current_state:
            updated_markdown += tc

        if 'unreleased' in current_state:
            unreleased.append((tt, hc, tc))

    if 'unreleased' in current_state and unreleased:
        changelog = _prepare_changelog(unreleased, new_version, git_tag)
        updated_markdown += changelog

    return (
        updated_markdown if changelog else "",
        changelog,
    )


def _prepare_changelog(
    tokens: List[Tuple[str, int, str]], new_version: str, git_tag: str
) -> str:
    current = ''
    previous = ''
    output = ''
    keyword_text = ''
    unreleased_link = ''

    for tt, _, tc in tokens:
        previous = current

        if tt == 'unreleased':
            if new_version:
                tc = __UNRELEASED_MATCHER.sub(new_version, tc)
                tc += ' - {}'.format(date.today().isoformat())
            output += tc
        elif tt == 'unreleased_link':
            if new_version:
                tc = __UNRELEASED_MATCHER.sub(new_version, tc)
                tc = __MASTER_MATCHER.sub(git_tag, tc)
            unreleased_link += tc + '\n\n'
        elif 'kw_' in tt:
            if keyword_text.strip().count('\n') > 0:
                output += keyword_text
            keyword_text = tc
            current = tt
        elif 'kw_' in previous:
            keyword_text += tc
        else:
            output += tc

    if keyword_text.strip().count('\n') > 0:
        output += keyword_text.strip() + '\n\n'

    output += unreleased_link

    return output


def __build_scanner():
    def token_handler(key: str):
        """
        generates a lambda for the regex scanner with a given key.

        This lambda will return a tuple: key, count # of token and token.

        The count is used to identify the level of heading on a special
        ended which can be used to identify when this section ended.
        """
        return lambda _, token: (key, token.count('#'), token)

    return re.Scanner(
        [
            (r'#{1,} [Aa]dded', token_handler('kw_added')),
            (r'#{1,} [Cc]hanged', token_handler('kw_changed')),
            (r'#{1,} [Dd]eprecated', token_handler('kw_deprecated')),
            (r'#{1,} [Rr]emoved', token_handler('kw_removed')),
            (r'#{1,} [Ff]ixed', token_handler('kw_fixed')),
            (r'#{1,} [Ss]ecurity', token_handler('kw_security')),
            (r'#{1,}.*(?=[Uu]nreleased).*', token_handler('unreleased')),
            (
                r'\[[Uu]nreleased\].*',
                token_handler('unreleased_link'),
            ),
            (r'#{1,} .*', token_handler('heading')),
            (r'\n', token_handler('newline')),
            (r'..*', token_handler('any')),
        ]
    )


__CHANGELOG_SCANNER = __build_scanner()


def _tokenize(
    markdown: str,
) -> List[Tuple[int, str, int, str],]:
    toks, remainder = __CHANGELOG_SCANNER.scan(markdown)
    if remainder != '':
        raise ChangelogError(
            "unrecognized tokens in markdown: {}".format(remainder)
        )

    return toks



# -*- coding: utf-8 -*-
# pontos/release/release.py
# Copyright (C) 2020 - 2021 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from typing import Callable, Dict, List, Tuple, Union
import json
import tempfile
from pathlib import Path
import shutil

import requests

from pontos import version


def build_release_dict(
    release_version: str,
    release_changelog: str,
    *,
    name: str = '',
    target_commitish: str = '',
    draft: bool = False,
    prerelease: bool = False,
) -> Dict[str, Union[str, bool]]:
    """
    builds the dict for release post on github, see:
    https://docs.github.com/en/rest/reference/repos#create-a-release
    for more details.

    Arguments:
        release_version: The version (str) that will be set
        release_changelog: content of the Changelog (str) for the release
        name: name (str) of the release, e.g. 'pontos 1.0.0'
        target_commitish: needed when tag is not there yet (str)
        draft: If the release is a draft (bool)
        prerelease: If the release is a pre release (bool)

    Returns:
        The dictionary containing the release information.
    """
    tag_name = (
        release_version
        if release_version.startswith('v')
        else "v" + release_version
    )
    return {
        'tag_name': tag_name,
        'target_commitish': target_commitish,
        'name': name,
        'body': release_changelog,
        'draft': draft,
        'prerelease': prerelease,
    }


def commit_files(
    filename: str,
    commit_msg: str,
    shell_cmd_runner: Callable,
    *,
    git_signing_key: str = '',
):
    """Add files to staged and commit staged files.

    filename: The filename of file to add and commit
    commit_msg: The commit message for the commit
    shell_cmd_runner:
    git_signing_key: The signing key to sign this commit

    Arguments:
        to: The version (str) that will be set
        develop: Wether to set version to develop or not (bool)

    Returns:
       executed: True if successfully executed, False else
       filename: The filename of the project definition
    """

    shell_cmd_runner(f"git add {filename}")
    shell_cmd_runner("git add *__version__.py || echo 'ignoring __version__'")
    shell_cmd_runner("git add CHANGELOG.md")
    shell_cmd_runner(
        f"git commit -S {git_signing_key} -m '{commit_msg}'",
    )


def download(
    url: str,
    filename: str,
    requests_module: requests,
    path: Path,
) -> Path:
    """Download file in url to filename

    Arguments:
        url: The url of the file we want to download
        filename: The name of the file to store the download in
        requests_module: the python request module
        path: the python pathlib.Path module

    Returns:
       Path to the downloaded file
    """

    file_path = path(tempfile.gettempdir()) / filename

    with requests_module.get(url, stream=True) as resp, file_path.open(
        mode='wb'
    ) as download_file:
        shutil.copyfileobj(resp.raw, download_file)

    return file_path


def get_project_name(
    shell_cmd_runner: Callable,
    *,
    remote: str = 'origin',
) -> str:
    """Get the git repository name"""
    # https://stackoverflow.com/a/42543006
    ret = shell_cmd_runner(f'git remote get-url {remote}')
    return ret.stdout.split('/')[-1].replace('.git', '').replace('\n', '')


def update_version(
    to: str, _version: version, *, develop: bool = False
) -> Tuple[bool, str]:
    """Use pontos-version to update the version.

    Arguments:
        to: The version (str) that will be set
        _version: Version module
        develop: Wether to set version to develop or not (bool)

    Returns:
       executed: True if successfully executed, False else
       filename: The filename of the project definition
    """
    args = ['--quiet']
    args.append('update')
    args.append(to)
    if develop:
        args.append('--develop')
    executed, filename = _version.main(leave=False, args=args)

    if not executed:
        if filename == "":
            print("No project definition found.")
        else:
            print(f"Unable to update version {to} in {filename}")

    return executed, filename


def upload_assets(
    username: str,
    token: str,
    pathnames: List[str],
    github_json: Dict,
    path: Path,
    requests_module: requests,
) -> bool:
    """Function to upload assets

    Arguments:
        username: The GitHub username to use for the upload
        token: That username's GitHub token
        pathnames: List of paths to asset files
        github_json: The github dictionary, containing relevant information
            for the uplaod
        path: the python pathlib.Path module
        requests_module: the python request module

    Returns:
        True on success, false else
    """
    print(f"Uploading assets: {pathnames}")

    asset_url = github_json['upload_url'].replace('{?name,label}', '')
    paths = [path(f'{p}.asc') for p in pathnames]

    headers = {
        'Accept': 'application/vnd.github.v3+json',
        'content-type': 'application/octet-stream',
    }
    auth = (username, token)

    for path in paths:
        to_upload = path.read_bytes()
        resp = requests_module.post(
            f"{asset_url}?name={path.name}",
            headers=headers,
            auth=auth,
            data=to_upload,
        )

        if resp.status_code != 201:
            print(
                f"Wrong response status {resp.status_code}"
                f" while uploading {path.name}"
            )
            print(json.dumps(resp.text, indent=4, sort_keys=True))
            return False
        else:
            print(f"uploaded: {path.name}")

    return True



# Copyright (C) 2020-2021 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import unittest

from pathlib import Path
from unittest.mock import MagicMock

from pontos.version import VersionCommand, VersionError


class VerifyVersionTestCase(unittest.TestCase):
    def test_current_version_not_pep440_compliant(self):
        fake_version_py = Path('foo.py')
        VersionCommand.get_current_version = MagicMock(return_value='1.02.03')
        cmd = VersionCommand(version_file_path=fake_version_py)

        with self.assertRaisesRegex(
            VersionError,
            'The version .* in foo.py is not PEP 440 compliant.',
        ):
            cmd.verify_version('1.2.3')

    def test_current_version_not_equal_pyproject_toml_version(self):
        fake_version_py = Path('foo.py')
        fake_path_class = MagicMock(spec=Path)
        fake_path = fake_path_class.return_value
        fake_path.read_text.return_value = '[tool.poetry]\nversion = "1.1.1"'

        VersionCommand.get_current_version = MagicMock(return_value='1.2.3')
        cmd = VersionCommand(
            version_file_path=fake_version_py,
            pyproject_toml_path=fake_path,
        )

        with self.assertRaisesRegex(
            VersionError,
            'The version .* in .* doesn\'t match the current version .*.',
        ):
            cmd.verify_version('1.2.3')

    def test_current_version(self):
        fake_version_py = Path('foo.py')
        fake_path_class = MagicMock(spec=Path)
        fake_path = fake_path_class.return_value
        fake_path.read_text.return_value = '[tool.poetry]\nversion = "1.2.3"'

        print_mock = MagicMock()
        VersionCommand.get_current_version = MagicMock(return_value='1.2.3')
        VersionCommand._print = print_mock  # pylint: disable=protected-access

        cmd = VersionCommand(
            version_file_path=fake_version_py,
            pyproject_toml_path=fake_path,
        )
        cmd.verify_version('current')

        print_mock.assert_called_with('OK')

    def test_provided_version_missmatch(self):
        fake_version_py = Path('foo.py')
        fake_path_class = MagicMock(spec=Path)
        fake_path = fake_path_class.return_value
        fake_path.read_text.return_value = '[tool.poetry]\nversion = "1.2.3"'

        VersionCommand.get_current_version = MagicMock(return_value='1.2.3')

        cmd = VersionCommand(
            version_file_path=fake_version_py,
            pyproject_toml_path=fake_path,
        )

        with self.assertRaisesRegex(
            VersionError,
            'Provided version .* does not match the current version .*.',
        ):
            cmd.verify_version('1.2.4')

    def test_verify_success(self):
        fake_version_py = Path('foo.py')
        fake_path_class = MagicMock(spec=Path)
        fake_path = fake_path_class.return_value
        fake_path.read_text.return_value = '[tool.poetry]\nversion = "1.2.3"'

        print_mock = MagicMock()
        VersionCommand.get_current_version = MagicMock(return_value='1.2.3')
        VersionCommand._print = print_mock  # pylint: disable=protected-access

        cmd = VersionCommand(
            version_file_path=fake_version_py,
            pyproject_toml_path=fake_path,
        )
        cmd.verify_version('1.2.3')

        print_mock.assert_called_with('OK')



# -*- coding: utf-8 -*-
# Copyright (C) 2020 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import unittest
from pathlib import Path
from unittest.mock import MagicMock
from pontos.version import CMakeVersionParser, VersionError, CMakeVersionCommand

# pylint: disable=W0212


class CMakeVersionCommandTestCase(unittest.TestCase):
    def test_raise_exception_file_not_exists(self):
        fake_path_class = MagicMock(spec=Path)
        fake_path = fake_path_class.return_value
        fake_path.__str__.return_value = 'CMakeLists.txt'
        fake_path.exists.return_value = False
        with self.assertRaises(VersionError):
            CMakeVersionCommand(cmake_lists_path=fake_path)

    def test_raise_exception_no_project(self):
        fake_path_class = MagicMock(spec=Path)
        fake_path = fake_path_class.return_value
        fake_path.__str__.return_value = 'CMakeLists.txt'
        fake_path.exists.return_value = True
        fake_path.read_text.return_value = ""
        with self.assertRaises(ValueError):
            CMakeVersionCommand(cmake_lists_path=fake_path).run(args=['show'])
        fake_path.read_text.assert_called_with()

    def test_return_error_string_incorrect_version_on_verify(self):
        fake_path_class = MagicMock(spec=Path)
        fake_path = fake_path_class.return_value
        fake_path.__str__.return_value = 'CMakeLists.txt'
        fake_path.exists.return_value = True
        fake_path.read_text.return_value = ""
        result = CMakeVersionCommand(cmake_lists_path=fake_path).run(
            args=['verify', 'so_much_version_so_much_wow']
        )
        self.assertTrue(
            isinstance(result, str), "expected result to be an error string"
        )

    def test_return_0_correct_version_on_verify(self):
        fake_path_class = MagicMock(spec=Path)
        fake_path = fake_path_class.return_value
        fake_path.__str__.return_value = 'CMakeLists.txt'
        fake_path.exists.return_value = True
        fake_path.read_text.return_value = ""
        result = CMakeVersionCommand(cmake_lists_path=fake_path).run(
            args=['verify', '21.4']
        )
        self.assertEqual(0, result)

    def test_should_call_print_current_version_without_raising_exception(self):
        fake_path_class = MagicMock(spec=Path)
        fake_path = fake_path_class.return_value
        fake_path.__str__.return_value = 'CMakeLists.txt'
        fake_path.exists.return_value = True
        fake_path.read_text.return_value = "project(VERSION 21)"
        CMakeVersionCommand(cmake_lists_path=fake_path).run(args=['show'])
        fake_path.read_text.assert_called_with()

    def test_raise_update_version(self):
        fake_path_class = MagicMock(spec=Path)
        fake_path = fake_path_class.return_value
        fake_path.__str__.return_value = 'CMakeLists.txt'
        fake_path.exists.return_value = True
        fake_path.read_text.return_value = (
            "project(VERSION 21)\nset(PROJECT_DEV_VERSION 0)"
        )
        CMakeVersionCommand(cmake_lists_path=fake_path).run(
            args=['update', '22', '--develop']
        )
        fake_path.read_text.assert_called_with()
        fake_path.write_text.assert_called_with(
            'project(VERSION 22)\nset(PROJECT_DEV_VERSION 1)'
        )


class CMakeVersionParserTestCase(unittest.TestCase):
    def test_get_current_version_single_line_project(self):
        under_test = CMakeVersionParser("project(VERSION 2.3.4)")
        self.assertEqual(under_test.get_current_version(), '2.3.4')

    def test_update_version_project(self):
        under_test = CMakeVersionParser("project(VERSION 2.3.4)")
        self.assertEqual(
            under_test.update_version('2.3.5'), "project(VERSION 2.3.5)"
        )

    def test_update_raise_exception_when_version_is_incorrect(self):
        under_test = CMakeVersionParser("project(VERSION 2.3.4)")
        with self.assertRaises(VersionError):
            under_test.update_version('so_much_version_so_much_wow')

    def test_not_confuse_version_outside_project(self):
        under_test = CMakeVersionParser(
            "non_project(VERSION 2.3.5)\nproject(VERSION 2.3.4)"
        )
        self.assertEqual(under_test.get_current_version(), '2.3.4')

    def test_get_current_version_multiline_project(self):
        under_test = CMakeVersionParser("project\n(\nVERSION\n\t    2.3.4)")
        self.assertEqual(under_test.get_current_version(), '2.3.4')

    def test_find_project_dev_version(self):
        test_cmake_lists = """
        project(
            DESCRIPTION something
            VERSION 41.41.41
            LANGUAGES c
        )
        set(
            PROJECT_DEV_VERSION 1
        )
        """
        under_test = CMakeVersionParser(test_cmake_lists)
        self.assertEqual(under_test._project_dev_version_line_number, 7)
        self.assertEqual(under_test._project_dev_version, '1')

    def test_update_project_dev_version(self):
        test_cmake_lists = """
        project(
            DESCRIPTION something
            VERSION 41.41.41
            LANGUAGES c
        )
        set(
            PROJECT_DEV_VERSION 1
        )
        """
        under_test = CMakeVersionParser(test_cmake_lists)

        self.assertEqual(under_test._project_dev_version_line_number, 7)
        self.assertEqual(under_test._project_dev_version, '1')
        result = under_test.update_version('41.41.41', develop=False)
        self.assertEqual(under_test._project_dev_version, '0')
        self.assertEqual(
            result,
            test_cmake_lists.replace(
                'PROJECT_DEV_VERSION 1', 'PROJECT_DEV_VERSION 0'
            ),
        )

    def test_update_project_dev_version_when_succeeded_by_another_set(self):
        test_cmake_lists = """
        cmake_minimum_required(VERSION 3.1)

        project(hello_world VERSION 41.41.41)
        set(PROJECT_DEV_VERSION 1)

        add_executable(app main.c)
        """
        under_test = CMakeVersionParser(test_cmake_lists)

        self.assertEqual(under_test._project_dev_version_line_number, 4)
        self.assertEqual(under_test._project_dev_version, '1')
        result = under_test.update_version('41.41.41', develop=False)
        self.assertEqual(under_test._project_dev_version, '0')
        self.assertEqual(
            result,
            test_cmake_lists.replace(
                'PROJECT_DEV_VERSION 1', 'PROJECT_DEV_VERSION 0'
            ),
        )

    def test_get_current_version_multiline_project_combined_token(self):
        under_test = CMakeVersionParser(
            "project\n(\nDESCRIPTION something VERSION 2.3.4 LANGUAGES c\n)"
        )
        self.assertEqual(under_test.get_current_version(), '2.3.4')

    def test_raise_exception_project_no_version(self):
        with self.assertRaises(ValueError) as context:
            CMakeVersionParser("project(DESCRIPTION something LANGUAGES c)")
        self.assertEqual(
            str(context.exception), 'unable to find cmake version in project.'
        )

    def test_raise_exception_no_project(self):
        with self.assertRaises(ValueError) as context:
            CMakeVersionParser(
                "non_project(VERSION 2.3.5)",
            )

        self.assertEqual(
            str(context.exception), 'unable to find cmake version.'
        )
